@@ -40,11 +40,10 @@ except ImportError:
     vc_version = 0
 
 # The tuple giving the version number.
-version_tuple = (7, 2, 2, vc_version)
+version_tuple = (7, 3, 2, vc_version)
 
 # The name of this version.
-version_name = "What's on the menu."
-# 7.3: The world (wide web) is not enough.
+version_name = "The world (wide web) is not enough."
 
 
 # A string giving the version number only (8.0.1.123).
@@ -551,6 +550,10 @@ def post_import():
         vars(renpy.exports).setdefault(k, v)
 
 
+def issubmodule(sub, module):
+    return sub == module or sub.startswith(module + ".")
+
+
 def reload_all():
     """
     Resets all modules to the state they were in right after import_all
@@ -588,10 +591,11 @@ def reload_all():
     renpy.display.interface = None
 
     py_compile_cache = renpy.python.py_compile_cache
+    reload_modules = renpy.config.reload_modules
 
     # Delete the store modules.
     for i in sys.modules.keys():
-        if i.startswith("store") or i == "renpy.store":
+        if issubmodule(i, "store") or i == "renpy.store":
             m = sys.modules[i]
 
             if m is not None:
@@ -599,6 +603,14 @@ def reload_all():
 
             del sys.modules[i]
 
+        elif any(issubmodule(i, m) for m in reload_modules):
+            m = sys.modules[i]
+
+            if m is not None:
+                m.__dict__.clear()
+
+            del sys.modules[i]
+
     # Restore the state of all modules from backup.
     backup.restore()
 
