@@ -1,4 +1,4 @@
-# Copyright 2004-2016 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2019 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -19,6 +19,7 @@
 # OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION
 # WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
 
+from __future__ import print_function
 import renpy.display
 import time
 import collections
@@ -32,6 +33,7 @@ profile_log = renpy.log.open("profile_screen", developer=True, append=False, flu
 # A map from screen name to ScreenProfile object.
 profile = { }
 
+
 class ScreenProfile(renpy.object.Object):
     """
     :doc: profile_screen
@@ -107,6 +109,7 @@ class ScreenProfile(renpy.object.Object):
                 name = tuple(name.split())
                 profile[name] = self
 
+
 def get_profile(name):
     """
     Returns the profile object for the screen with `name`, or a default
@@ -126,10 +129,12 @@ def get_profile(name):
 
 # Cache ########################################################################
 
+
 # A map from screen name to a list of ScreenCache objects. We ensure the cache
 # does not exceed config.screen_cache_size for each screen.
 predict_cache = collections.defaultdict(list)
 
+
 class ScreenCache(object):
     """
     Represents an entry in the screen cache. Upon creation, puts itself into
@@ -152,8 +157,10 @@ class ScreenCache(object):
         if len(pc) > renpy.config.screen_cache_size:
             pc.pop(0)
 
+
 cache_put = ScreenCache
 
+
 def cache_get(screen, args, kwargs):
     """
     Returns the cache to use when `screen` is accessed with `args` and
@@ -190,6 +197,8 @@ class Screen(renpy.object.Object):
     This class stores information about the screen.
     """
 
+    sensitive = "True"
+
     def __init__(self,
                  name,
                  function,
@@ -200,7 +209,8 @@ class Screen(renpy.object.Object):
                  variant=None,
                  parameters=False,
                  location=None,
-                 layer="screens"):
+                 layer="screens",
+                 sensitive="True"):
 
         # The name of this screen.
         if isinstance(name, basestring):
@@ -220,7 +230,7 @@ class Screen(renpy.object.Object):
 
         # If this is a SL2 screen, the SLScreen node at the root of this
         # screen.
-        if isinstance(function, renpy.sl2.slast.SLScreen): # @UndefinedVariable
+        if isinstance(function, renpy.sl2.slast.SLScreen):  # @UndefinedVariable
             self.ast = function
         else:
             self.ast = None
@@ -249,6 +259,9 @@ class Screen(renpy.object.Object):
         # The layer the screen will be shown on.
         self.layer = layer
 
+        # Is this screen sensitive? An expression.
+        self.sensitive = sensitive
+
         global prepared
         global analyzed
 
@@ -257,7 +270,7 @@ class Screen(renpy.object.Object):
 
 
 # Phases we can be in.
-PREDICT = 0 # Predicting the screen before it is shown.
+PREDICT = 0  # Predicting the screen before it is shown.
 SHOW = 1    # Showing the screen for the first time.
 UPDATE = 2  # Showing the screen for the second and later times.
 HIDE = 3    # After the screen has been hid with "hide screen" (or the end of call screen).
@@ -271,16 +284,31 @@ phase_name = [
     "OLD",
     ]
 
+
 class ScreenDisplayable(renpy.display.layout.Container):
     """
     A screen is a collection of widgets that are displayed together. This
     class is responsible for managing the display of a screen.
     """
 
-    nosave = [ 'screen', 'child', 'children', 'transforms', 'widgets', 'old_widgets', 'hidden_widgets', 'old_transforms', 'cache', 'profile', 'phase', 'use_cache' ]
+    nosave = [
+        'screen',
+        'child',
+        'children',
+        'transforms',
+        'widgets',
+        'old_widgets',
+        'hidden_widgets',
+        'old_transforms',
+        'cache',
+        'miss_cache',
+        'profile',
+        'phase',
+        'use_cache' ]
 
     restarting = False
     hiding = False
+    transient = False
 
     def after_setstate(self):
         self.screen = get_screen_variant(self.screen_name[0])
@@ -294,10 +322,11 @@ class ScreenDisplayable(renpy.display.layout.Container):
         self.cache = { }
         self.phase = UPDATE
         self.use_cache = { }
+        self.miss_cache = { }
 
         self.profile = profile.get(self.screen_name, None)
 
-    def __init__(self, screen, tag, layer, widget_properties={}, scope={}, **properties):
+    def __init__(self, screen, tag, layer, widget_properties={}, scope={}, transient=False, **properties):
 
         super(ScreenDisplayable, self).__init__(**properties)
 
@@ -354,6 +383,10 @@ class ScreenDisplayable(renpy.display.layout.Container):
         else:
             self.use_cache = { }
 
+        # A version of the cache that's used when we have a screen that is
+        # being displayed with the same tag with a cached copy of the screen
+        # we want to display.
+        self.miss_cache = { }
 
         # What widgets and transforms were the last time this screen was
         # updated. Used to communicate with the ui module, and only
@@ -377,6 +410,9 @@ class ScreenDisplayable(renpy.display.layout.Container):
         self.restarting = False
         self.hiding = False
 
+        # Is this a transient screen?
+        self.transient = transient
+
         # Modal and zorder.
         self.modal = renpy.python.py_eval(self.screen.modal, locals=self.scope)
         self.zorder = renpy.python.py_eval(self.screen.zorder, locals=self.scope)
@@ -390,12 +426,12 @@ class ScreenDisplayable(renpy.display.layout.Container):
     def visit(self):
         return [ self.child ]
 
-    def visit_all(self, callback):
+    def visit_all(self, callback, seen=None):
         callback(self)
 
         try:
             push_current_screen(self)
-            self.child.visit_all(callback)
+            self.child.visit_all(callback, seen=None)
         finally:
             pop_current_screen()
 
@@ -428,12 +464,25 @@ class ScreenDisplayable(renpy.display.layout.Container):
 
         return rv
 
+    def _handles_event(self, event):
+        if self.child is None:
+
+            if self.transient:
+                return False
+
+            self.update()
+
+        return self.child._handles_event(event)
+
     def _hide(self, st, at, kind):
 
         if self.phase == HIDE:
             hid = self
         else:
 
+            if (self.child is not None) and (not self.child._handles_event(kind)):
+                return None
+
             updated_screens.discard(self)
             self.update()
 
@@ -443,6 +492,9 @@ class ScreenDisplayable(renpy.display.layout.Container):
             if self.child is None:
                 return None
 
+            if not self.child._handles_event(kind):
+                return None
+
             if self.screen.ast is not None:
                 self.screen.ast.copy_on_change(self.cache.get(0, {}))
 
@@ -461,7 +513,7 @@ class ScreenDisplayable(renpy.display.layout.Container):
             return None
 
         renpy.ui.detached()
-        hid.child = renpy.ui.fixed(focus="_screen_" + "_".join(self.screen_name))
+        hid.child = renpy.ui.default_fixed(focus="_screen_" + "_".join(self.screen_name))
         hid.children = [ hid.child ]
         renpy.ui.close()
 
@@ -479,7 +531,7 @@ class ScreenDisplayable(renpy.display.layout.Container):
 
         return rv
 
-    def _in_old_scene(self):
+    def _in_current_store(self):
 
         if self.screen is None:
             return self
@@ -495,8 +547,9 @@ class ScreenDisplayable(renpy.display.layout.Container):
 
         rv = self.copy()
         rv.phase = OLD
-        return rv
+        rv.child = self.child._in_current_store()
 
+        return rv
 
     def update(self):
 
@@ -533,16 +586,15 @@ class ScreenDisplayable(renpy.display.layout.Container):
 
             if profile:
                 profile_log.write("%s %s %s",
-                    phase_name[self.phase],
-                    " ".join(self.screen_name),
-                    datetime.datetime.now().strftime("%H:%M:%S.%f"))
+                                  phase_name[self.phase],
+                                  " ".join(self.screen_name),
+                                  datetime.datetime.now().strftime("%H:%M:%S.%f"))
 
                 start = time.time()
 
                 if self.profile.debug:
                     debug = True
 
-
         # Cycle widgets and transforms.
         self.old_widgets = self.widgets
         self.old_transforms = self.transforms
@@ -563,7 +615,7 @@ class ScreenDisplayable(renpy.display.layout.Container):
         try:
 
             renpy.ui.detached()
-            self.child = renpy.ui.fixed(focus="_screen_" + "_".join(self.screen_name))
+            self.child = renpy.ui.default_fixed(focus="_screen_" + "_".join(self.screen_name))
             self.children = [ self.child ]
 
             self.scope["_scope"] = self.scope
@@ -575,6 +627,8 @@ class ScreenDisplayable(renpy.display.layout.Container):
             renpy.ui.close()
 
         finally:
+            del self.scope["_scope"]
+
             renpy.ui.screen = old_ui_screen
             pop_current_screen()
 
@@ -583,6 +637,9 @@ class ScreenDisplayable(renpy.display.layout.Container):
         self.old_transforms = None
         self.old_transfers = True
 
+        if self.miss_cache:
+            self.miss_cache.clear()
+
         # Deal with the case where the screen version changes.
         if (self.cache.get(NAME, None) is not old_cache) and (self.current_transform_event is None) and (self.phase == UPDATE):
             self.current_transform_event = "update"
@@ -603,9 +660,6 @@ class ScreenDisplayable(renpy.display.layout.Container):
             if self.profile.debug:
                 profile_log.write("\n")
 
-        if self.phase == SHOW:
-            self.phase = UPDATE
-
         return self.widgets
 
     def render(self, w, h, st, at):
@@ -613,6 +667,9 @@ class ScreenDisplayable(renpy.display.layout.Container):
         if not self.child:
             self.update()
 
+        if self.phase == SHOW:
+            self.phase = UPDATE
+
         try:
             push_current_screen(self)
             child = renpy.display.render.render(self.child, w, h, st, at)
@@ -623,8 +680,9 @@ class ScreenDisplayable(renpy.display.layout.Container):
         rv.focus_screen = self
 
         hiding = (self.phase == OLD) or (self.phase == HIDE)
+        sensitive = renpy.python.py_eval(self.screen.sensitive, locals=self.scope)
 
-        rv.blit(child, (0, 0), focus=not hiding, main=not hiding)
+        rv.blit(child, (0, 0), focus=sensitive and not hiding, main=not hiding)
         rv.modal = self.modal and not hiding
 
         return rv
@@ -640,6 +698,9 @@ class ScreenDisplayable(renpy.display.layout.Container):
         if (self.phase == OLD) or (self.phase == HIDE):
             return
 
+        if not renpy.python.py_eval(self.screen.sensitive, locals=self.scope):
+            ev = renpy.display.interface.time_event
+
         try:
             push_current_screen(self)
 
@@ -656,6 +717,7 @@ class ScreenDisplayable(renpy.display.layout.Container):
     def get_phase_name(self):
         return phase_name[self.phase]
 
+
 # The name of the screen that is currently being displayed, or
 # None if no screen is being currently displayed.
 _current_screen = None
@@ -663,14 +725,18 @@ _current_screen = None
 # The stack of old current screens.
 current_screen_stack = [ ]
 
+
 def push_current_screen(screen):
     global _current_screen
     current_screen_stack.append(_current_screen)
     _current_screen = screen
 
+
 def pop_current_screen():
+    global _current_screen
     _current_screen = current_screen_stack.pop()
 
+
 # A map from (screen_name, variant) tuples to screen.
 screens = { }
 
@@ -680,6 +746,7 @@ screens_by_name = collections.defaultdict(dict)
 # The screens that were updated during the current interaction.
 updated_screens = set()
 
+
 def get_screen_variant(name, candidates=None):
     """
     Get a variant screen object for `name`.
@@ -698,6 +765,7 @@ def get_screen_variant(name, candidates=None):
 
     return None
 
+
 def get_all_screen_variants(name):
     """
     Gets all variants of the screen with `name`.
@@ -714,6 +782,7 @@ def get_all_screen_variants(name):
 
     return rv
 
+
 # Have all screens been analyzed?
 analyzed = False
 
@@ -724,11 +793,16 @@ prepared = False
 sorted_screens = [ ]
 screens_at_sort = { }
 
+# The list of screens that participate in a use cycle.
+use_cycle = [ ]
+
+
 def sort_screens():
     """
     Produces a list of SL2 screens in topologically sorted order.
     """
 
+    global use_cycle
     global sorted_screens
     global screens_at_sort
 
@@ -741,6 +815,8 @@ def sort_screens():
     # For each screen, the set of screens that use it.
     reverse = collections.defaultdict(set)
 
+    names = { i[0] for i in screens }
+
     for k, v in screens.items():
 
         name = k[0]
@@ -752,6 +828,10 @@ def sort_screens():
             continue
 
         def callback(uses):
+
+            if uses not in names:
+                return
+
             depends[name].add(uses)
             reverse[uses].add(name)
 
@@ -774,14 +854,16 @@ def sort_screens():
 
         del reverse[name]
 
-    # A use-cycle is possible, but we live with it - we need to return something
-    # from this in order to be able to show error messages.
+    # Store the use cycle for later reporting.
+    use_cycle = reverse.keys()
+    use_cycle.sort()
 
     sorted_screens = rv
     screens_at_sort = dict(screens)
 
     return rv
 
+
 def sorted_variants():
     """
     Produces a list of screen variants in topological order.
@@ -794,6 +876,7 @@ def sorted_variants():
 
     return rv
 
+
 def analyze_screens():
     """
     Analyzes all screens.
@@ -812,6 +895,7 @@ def analyze_screens():
 
     analyzed = True
 
+
 def prepare_screens():
     """
     Prepares all screens for use.
@@ -836,6 +920,10 @@ def prepare_screens():
 
     prepared = True
 
+    if renpy.config.developer and use_cycle:
+        raise Exception("The following screens use each other in a loop: " + ", ".join(use_cycle) +". This is not allowed.")
+
+
 def define_screen(*args, **kwargs):
     """
     :doc: screens
@@ -906,8 +994,7 @@ def get_screen(name, layer=None):
     This can also take a list of names, in which case the first screen
     that is showing is returned.
 
-    This function can be used to check if a screen is showing, using code
-    like::
+    This function can be used to check if a screen is showing::
 
         if renpy.get_screen("say"):
             text "The say screen is showing."
@@ -930,7 +1017,9 @@ def get_screen(name, layer=None):
         if sd is not None:
             return sd
 
-        sd = sl.get_displayable_by_name(layer, name)
+    for tag in name:
+
+        sd = sl.get_displayable_by_name(layer, (tag, ))
         if sd is not None:
             return sd
 
@@ -953,6 +1042,7 @@ def has_screen(name):
     else:
         return False
 
+
 def show_screen(_screen_name, *_args, **kwargs):
     """
     :doc: screens
@@ -962,13 +1052,17 @@ def show_screen(_screen_name, *_args, **kwargs):
     Shows the named screen. This takes the following keyword arguments:
 
     `_screen_name`
-        The name of the  screen to show.
+        The name of the screen to show.
     `_layer`
         The layer to show the screen on.
+    `_zorder`
+        The zorder to show the screen on. If not specified, defaults to
+        the zorder associated with the screen. It that's not specified,
+        it is 0 by default.
     `_tag`
         The tag to show the screen with. If not specified, defaults to
         the tag associated with the screen. It that's not specified,
-        defaults to the name of the screen.,
+        defaults to the name of the screen.
     `_widget_properties`
         A map from the id of a widget to a property name -> property
         value map. When a widget with that id is shown by the screen,
@@ -985,6 +1079,7 @@ def show_screen(_screen_name, *_args, **kwargs):
     _tag = kwargs.pop("_tag", None)
     _widget_properties = kwargs.pop("_widget_properties", {})
     _transient = kwargs.pop("_transient", False)
+    _zorder = kwargs.pop("_zorder", None)
 
     name = _screen_name
 
@@ -1010,12 +1105,16 @@ def show_screen(_screen_name, *_args, **kwargs):
     else:
         scope.update(kwargs)
 
-    d = ScreenDisplayable(screen, _tag, _layer, _widget_properties, scope)
+    d = ScreenDisplayable(screen, _tag, _layer, _widget_properties, scope, transient=_transient)
 
-    old_d = get_screen(name, _layer)
+    if _zorder is None:
+        _zorder = d.zorder
+
+    old_d = get_screen(_tag, _layer)
 
     if old_d and old_d.cache:
         d.cache = old_d.cache
+        d.miss_cache = cache_get(screen, _args, kwargs)
         d.phase = UPDATE
     else:
         d.cache = cache_get(screen, _args, kwargs)
@@ -1023,7 +1122,7 @@ def show_screen(_screen_name, *_args, **kwargs):
 
     sls = renpy.display.core.scene_lists()
 
-    sls.add(_layer, d, _tag, zorder=d.zorder, transient=_transient, keep_st=True, name=name)
+    sls.add(_layer, d, _tag, zorder=_zorder, transient=_transient, keep_st=True, name=name)
 
 
 def predict_screen(_screen_name, *_args, **kwargs):
@@ -1090,9 +1189,12 @@ def predict_screen(_screen_name, *_args, **kwargs):
         if renpy.config.debug_image_cache:
             import traceback
 
-            print "While predicting screen", _screen_name
+            print("While predicting screen", _screen_name)
             traceback.print_exc()
 
+    finally:
+        del scope["_scope"]
+
     renpy.ui.reset()
 
 
@@ -1113,6 +1215,7 @@ def hide_screen(tag, layer=None):
     if screen is not None:
         renpy.exports.hide(screen.tag, layer=layer)
 
+
 def use_screen(_screen_name, *_args, **kwargs):
 
     _name = kwargs.pop("_name", ())
@@ -1142,14 +1245,19 @@ def use_screen(_screen_name, *_args, **kwargs):
     scope["_scope"] = scope
     scope["_name"] = (_name, name)
 
-    screen.function(**scope)
+    try:
+        screen.function(**scope)
+    finally:
+        del scope["_scope"]
 
     _current_screen.old_transfers = old_transfers
 
+
 def current_screen():
     return _current_screen
 
-def get_widget(screen, id, layer=None): #@ReservedAssignment
+
+def get_widget(screen, id, layer=None):  # @ReservedAssignment
     """
     :doc: screens
 
@@ -1178,18 +1286,18 @@ def get_widget(screen, id, layer=None): #@ReservedAssignment
     rv = screen.widgets.get(id, None)
     return rv
 
-def get_widget_properties(id, screen=None, layer=None): # @ReservedAssignment
+
+def get_widget_properties(id, screen=None, layer=None):  # @ReservedAssignment
     """
     :doc: screens
 
     Returns the properties for the widget with `id` in the `screen`
     on `layer`. If `screen` is None, returns the properties for the
-    current screen.
-
-    This can be used inside screen code.
+    current screen. This can be used from Python or property code inside
+    a screen.
 
     Note that this returns a dictionary containing the widget properties,
-    and so to get an individual property
+    and so to get an individual property, the dictionary must be accessed.
     """
 
     if screen is None:
@@ -1210,6 +1318,7 @@ def get_widget_properties(id, screen=None, layer=None): # @ReservedAssignment
 
     return rv
 
+
 def before_restart():
     """
     This is called before Ren'Py restarts to put the screens into restart
@@ -1251,3 +1360,20 @@ def show_overlay_screens(suppress_overlay):
         for i in renpy.config.overlay_screens:
             if get_screen(i) is not None:
                 hide_screen(i)
+
+
+def per_frame():
+    """
+    Called from interact once per frame to invalidate screens we want to
+    update once per frame.
+    """
+
+    for i in renpy.config.per_frame_screens:
+        s = get_screen(i)
+
+        if s is None:
+            continue
+
+        updated_screens.discard(s)
+        renpy.display.render.invalidate(s)
+        s.update()
