@@ -1,4 +1,4 @@
-# Copyright 2004-2018 Tom Rothamel <pytom@bishoujo.us>
+# Copyright 2004-2019 Tom Rothamel <pytom@bishoujo.us>
 #
 # Permission is hereby granted, free of charge, to any person
 # obtaining a copy of this software and associated documentation files
@@ -56,7 +56,7 @@ import pygame_sdl2
 
 from renpy.text.extras import ParameterizedText, filter_text_tags
 from renpy.text.font import register_sfont, register_mudgefont, register_bmfont
-from renpy.text.text import language_tailor
+from renpy.text.text import language_tailor, BASELINE
 from renpy.display.behavior import Keymap
 from renpy.display.behavior import run, run as run_action, run_unhovered, run_periodic
 from renpy.display.behavior import map_event, queue_event, clear_keymap_cache
@@ -114,6 +114,8 @@ from renpy.ast import eval_who
 
 from renpy.loader import add_python_directory
 
+from renpy.lint import try_compile, try_eval
+
 renpy_pure("ParameterizedText")
 renpy_pure("Keymap")
 renpy_pure("has_screen")
@@ -182,6 +184,7 @@ def public_api():
     eval_who
     is_selected, is_sensitive
     add_python_directory
+    try_compile, try_eval
 
 
 del public_api
@@ -228,7 +231,7 @@ def in_rollback():
     Returns true if the game has been rolled back.
     """
 
-    return renpy.game.log.in_rollback()
+    return renpy.game.log.in_rollback() or renpy.game.after_rollback
 
 
 def can_rollback():
@@ -264,6 +267,8 @@ def checkpoint(data=None, keep_rollback=None):
     this function has been called, there should be no more interaction with the
     user in the current statement.
 
+    This will also clear the current screenshot used by saved games.
+
     `data`
         This data is returned by :func:`renpy.roll_forward_info` when the
         game is being rolled back.
@@ -274,6 +279,9 @@ def checkpoint(data=None, keep_rollback=None):
 
     renpy.game.log.checkpoint(data, keep_rollback=keep_rollback, hard=renpy.store._rollback)
 
+    if renpy.store._rollback and renpy.config.auto_clear_screenshot:
+        renpy.game.interface.clear_screenshot = True
+
 
 def block_rollback(purge=False):
     """
@@ -511,20 +519,20 @@ def get_hidden_tags(layer='master'):
     return renpy.game.context().images.get_hidden_tags(layer)
 
 
-def get_attributes(tag, layer=None):
+def get_attributes(tag, layer=None, if_hidden=None):
     """
     :doc: image_func
 
     Return a tuple giving the image attributes for the image `tag`. If
     the image tag has not had any attributes associated since the last
-    time it was hidden, returns None.
+    time it was hidden, returns `if_hidden`.
 
     `layer`
         The layer to check. If None, uses the default layer for `tag`.
     """
 
     layer = default_layer(layer, tag)
-    return renpy.game.context().images.get_attributes(layer, tag, None)
+    return renpy.game.context().images.get_attributes(layer, tag, if_hidden)
 
 
 def predict_show(name, layer=None, what=None, tag=None, at_list=[ ]):
@@ -800,8 +808,14 @@ def input(prompt, default='', allow=None, exclude='{}', length=None, with_none=N
     `pixel_width`
         If not None, the input is limited to being this many pixels wide,
         in the font used by the input to display text.
+
+    If :var:`config.disable_input` is True, this function only returns
+    `default`.
     """
 
+    if renpy.config.disable_input:
+        return default
+
     renpy.exports.mode('input')
 
     roll_forward = renpy.exports.roll_forward_info()
@@ -856,7 +870,26 @@ def input(prompt, default='', allow=None, exclude='{}', length=None, with_none=N
     return rv
 
 
-def menu(items, set_expr):
+# The arguments and keyword arguments for the current menu call.
+menu_args = None
+menu_kwargs = None
+
+
+def get_menu_args():
+    """
+    :other:
+
+    Returns a tuple giving the arguments (as a tuple) and the keyword arguments
+    (as a dict) passed to the current menu statement.
+    """
+
+    if menu_args is None:
+        return tuple(), dict()
+
+    return menu_args, menu_kwargs
+
+
+def menu(items, set_expr, args=None, kwargs=None, item_arguments=None):
     """
     :undocumented:
 
@@ -864,6 +897,17 @@ def menu(items, set_expr):
     choice. Also handles conditions and the menuset.
     """
 
+    global menu_args
+    global menu_kwargs
+
+    args = args or tuple()
+    kwargs = kwargs or dict()
+
+    nvl = kwargs.pop("nvl", False)
+
+    if renpy.config.menu_arguments_callback is not None:
+        args, kwargs = renpy.config.menu_arguments_callback(*args, **kwargs)
+
     if renpy.config.old_substitutions:
         def substitute(s):
             return s % tag_quoting_dict
@@ -871,39 +915,86 @@ def menu(items, set_expr):
         def substitute(s):
             return s
 
-    # Filter the list of items to only include ones for which the
-    # condition is true.
-    items = [ (substitute(label), value)
-              for label, condition, value in items
-              if renpy.python.py_eval(condition) ]
+    if item_arguments is None:
+        item_arguments = [ (tuple(), dict()) ] * len(items)
 
     # Filter the list of items on the set_expr:
     if set_expr:
         set = renpy.python.py_eval(set_expr)  # @ReservedAssignment
-        items = [ (label, value)
-                  for label, value in items
-                  if label not in set ]
+
+        new_items = [ ]
+        new_item_arguments = [ ]
+
+        for i, ia in zip(items, item_arguments):
+            if i[0] not in set:
+                new_items.append(i)
+                new_item_arguments.append(ia)
+
+        items = new_items
+        item_arguments = new_item_arguments
     else:
         set = None  # @ReservedAssignment
 
+    # Filter the list of items to only include ones for which the
+    # condition is true.
+
+    if renpy.config.menu_actions:
+
+        location=renpy.game.context().current
+
+        new_items = [ ]
+
+        for (label, condition, value), (item_args, item_kwargs) in zip(items, item_arguments):
+            label = substitute(label)
+            condition = renpy.python.py_eval(condition)
+
+            if (not renpy.config.menu_include_disabled) and (not condition):
+                continue
+
+            if value is not None:
+                new_items.append((label, renpy.ui.ChoiceReturn(label, value, location, sensitive=condition, args=item_args, kwargs=item_kwargs)))
+            else:
+                new_items.append((label, None))
+
+    else:
+
+        new_items = [ (substitute(label), value)
+                      for label, condition, value in items
+                      if renpy.python.py_eval(condition) ]
+
     # Check to see if there's at least one choice in set of items:
-    choices = [ value for label, value in items if value is not None ]
+    choices = [ value for label, value in new_items if value is not None ]
 
     # If not, bail out.
     if not choices:
         return None
 
     # Show the menu.
-    rv = renpy.store.menu(items)
+    try:
+        old_menu_args = menu_args
+        old_menu_kwargs = menu_kwargs
+
+        menu_args = args
+        menu_kwargs = kwargs
+
+        if nvl:
+            rv = renpy.store.nvl_menu(new_items)  # @UndefinedVariable
+        else:
+            rv = renpy.store.menu(new_items)
+
+    finally:
+        menu_args = old_menu_args
+        old_menu_kwargs = old_menu_kwargs
 
     # If we have a set, fill it in with the label of the chosen item.
     if set is not None and rv is not None:
-        for label, value in items:
+        for label, condition, value in items:
             if value == rv:
                 try:
                     set.append(label)
                 except AttributeError:
                     set.add(label)
+
     return rv
 
 
@@ -990,16 +1081,33 @@ def display_menu(items,
     `screen`
         The name of the screen used to display the menu.
 
+    Note that most Ren'Py games do not use menu captions, but use narration
+    instead. To display a menu using narration, write::
+
+        $ narrator("Which direction would you like to go?", interact=False)
+        $ result = renpy.display_menu([ ("East", "east"), ("West", "west") ])
+
     """
 
+    menu_args, menu_kwargs = get_menu_args()
+    screen = menu_kwargs.pop("screen", screen)
+    with_none = menu_kwargs.pop("_with_none", with_none)
+    mode = menu_kwargs.pop("_mode", type)
+
     if interact:
-        renpy.exports.mode(type)
+        renpy.exports.mode(mode)
         choice_for_skipping()
 
-    # The possible choices in the menu.
-    choices = [ val for label, val in items ]
-    while None in choices:
-        choices.remove(None)
+    choices = [ ]
+
+    for _, val in items:
+        if isinstance(val, renpy.ui.ChoiceReturn):
+            val = val.value
+
+        if val is None:
+            continue
+
+        choices.append(val)
 
     # Roll forward.
     roll_forward = renpy.exports.roll_forward_info()
@@ -1020,6 +1128,10 @@ def display_menu(items,
     if in_fixed_rollback() and renpy.config.fix_rollback_without_choice:
         renpy.ui.saybehavior()
 
+    scope = dict(scope)
+
+    scope.update(menu_kwargs)
+
     # Show the menu.
     if has_screen(screen):
 
@@ -1035,12 +1147,23 @@ def display_menu(items,
             if not label:
                 value = None
 
-            if value is not None:
+            if isinstance(value, renpy.ui.ChoiceReturn):
+                action = value
+                chosen = action.get_chosen()
+                item_args = action.args
+                item_kwargs = action.kwargs
+
+            elif value is not None:
                 action = renpy.ui.ChoiceReturn(label, value, location)
                 chosen = action.get_chosen()
+                item_args = ()
+                item_kwargs = { }
+
             else:
                 action = None
                 chosen = False
+                item_args = ()
+                item_kwargs = { }
 
             if renpy.config.choice_screen_chosen:
                 me = MenuEntry((label, action, chosen))
@@ -1050,10 +1173,19 @@ def display_menu(items,
             me.caption = label
             me.action = action
             me.chosen = chosen
+            me.args = item_args
+            me.kwargs = item_kwargs
 
             item_actions.append(me)
 
-            show_screen(screen, items=item_actions, _widget_properties=props, _transient=True, _layer=renpy.config.choice_layer, **scope)
+            show_screen(
+                screen,
+                items=item_actions,
+                _widget_properties=props,
+                _transient=True,
+                _layer=renpy.config.choice_layer,
+                *menu_args,
+                **scope)
 
     else:
         renpy.ui.window(style=window_style, focus="menu")
@@ -1068,7 +1200,8 @@ def display_menu(items,
                       choice_chosen_button_style=choice_chosen_button_style,
                       **kwargs)
 
-    renpy.exports.shown_window()
+    if renpy.config.menu_showed_window:
+        renpy.exports.shown_window()
 
     # Log the chosen choice.
     for label, val in items:
@@ -1271,6 +1404,7 @@ def imagemap(ground, selected, hotspots, unselected=None, overlays=False,
 def pause(delay=None, music=None, with_none=None, hard=False, checkpoint=None):
     """
     :doc: other
+    :args: (delay=None, hard=False)
 
     Causes Ren'Py to pause. Returns true if the user clicked to end the pause,
     or false if the pause timed out or was skipped.
@@ -1278,20 +1412,23 @@ def pause(delay=None, music=None, with_none=None, hard=False, checkpoint=None):
     `delay`
         If given, the number of seconds Ren'Py should pause for.
 
-    `music`
-        Retained for compatibility purposes.
-
-    `with_none`
-        Determines if a with None clause is executed at the end of the pause.
-
     `hard`
-        If true, a click will not interrupt the pause. Use this sparingly,
-        as it's hard to distinguish a hard pause from a crashing game.
+        This must be given as a keyword argument. When True, Ren'Py may prevent
+        the user from clicking to interrupt the pause. If the player enables
+        skipping, the hard pause will be skipped. There may be other circumstances
+        where the hard pause ends early or prevents Ren'Py from operating properly,
+        these will not be treated as bugs.
 
-    `checkpoint`
-        If true, a checkpoint will be set, and players will be able to roll
-        back to this statement. If false, no checkpoint will be set. If None,
-        a checkpoint will only be set if delay is set.
+        In general, using hard pauses is rude. When the user clicks to advance
+        the game, it's an explicit request - the user wishes the game to advance.
+        To override that request is to assume you understand what the player
+        wants more than the player does.
+
+        Calling renpy.pause guarantees that whatever is on the screen will be
+        displayed for at least one frame, and hence has been shown to the
+        player.
+
+        tl;dr - Don't use renpy.pause with hard=True.
     """
 
     if checkpoint is None:
@@ -1349,12 +1486,12 @@ def movie_cutscene(filename, delay=None, loops=0, stop_music=True):
     """
     :doc: movie_cutscene
 
-    This displays an MPEG-1 cutscene for the specified number of
+    This displays a movie cutscene for the specified number of
     seconds. The user can click to interrupt the cutscene.
     Overlays and Underlays are disabled for the duration of the cutscene.
 
     `filename`
-        The name of a file containing an MPEG-1 movie.
+        The name of a file containing any movie playable by Ren'Py.
 
     `delay`
         The number of seconds to wait before ending the cutscene.
@@ -1611,6 +1748,10 @@ def reload_script():
 
     s = get_screen("menu")
 
+    session.pop("_reload_screen", None)
+    session.pop("_reload_screen_args", None)
+    session.pop("_reload_screen_kwargs", None)
+
     if not renpy.store.main_menu:
 
         if s is not None:
@@ -1698,13 +1839,14 @@ def call(label, *args, **kwargs):
     raise renpy.game.CallException(label, args, kwargs, from_current=from_current)
 
 
-def return_statement():
+def return_statement(value=None):
     """
     :doc: se_call
 
     Causes Ren'Py to return from the current Ren'Py-level call.
     """
 
+    renpy.store._return = value
     jump("_renpy_return")
 
 
@@ -2187,8 +2329,10 @@ def free_memory():
     Attempts to free some memory. Useful before running a renpygame-based
     minigame.
     """
+
     force_full_redraw()
     renpy.display.interface.kill_textures_and_surfaces()
+    renpy.text.font.free_memory()
 
 
 @renpy_pure
@@ -2252,6 +2396,14 @@ def pop_error_handler():
 
 
 def error(msg):
+    """
+    :doc: lint
+
+    Reports `msg`, a string, as as error for the user. This is logged as a
+    parse or lint error when approprate, and otherwise it is raised as an
+    exception.
+    """
+
     _error_handlers[-1](msg)
 
 
@@ -2477,6 +2629,18 @@ class placement(renpy.python.RevertableObject):
         self.yoffset = p[5]
         self.subpixel = p[6]
 
+    @property
+    def pos(self):
+        return self.xpos, self.ypos
+
+    @property
+    def anchor(self):
+        return self.xanchor, self.yanchor
+
+    @property
+    def offset(self):
+        return self.xoffset, self.yoffset
+
 
 def get_placement(d):
     """
@@ -2489,11 +2653,14 @@ def get_placement(d):
     This returns an object with the following fields, each corresponding to a style
     property:
 
+    * pos
     * xpos
-    * xanchor
-    * xoffset
     * ypos
+    * anchor
+    * xanchor
     * yanchor
+    * offset
+    * xoffset
     * yoffset
     * subpixel
 
@@ -2706,9 +2873,15 @@ def call_screen(_screen_name, *args, **kwargs):
 
     If the keyword argument `_with_none` is false, "with None" is not
     run at the end of end of the interaction.
+
+    If the keyword argument `_mode` in kwargs, it will be mode of this
+    interaction, otherwise it will be "screen" mode.
     """
 
-    renpy.exports.mode('screen')
+    mode = "screen"
+    if "_mode" in kwargs:
+        mode = kwargs.pop("_mode")
+    renpy.exports.mode(mode)
 
     with_none = renpy.config.implicit_with_none
 
@@ -2857,6 +3030,19 @@ def notify(message):
 
     Only one notification is displayed at a time. If a second notification
     is displayed, the first notification is replaced.
+
+    This function just calls :var:`config.notify`, allowing its implementation
+    to be replaced by assigning a new function to that variable.
+    """
+
+    renpy.config.notify(message)
+
+
+def display_notify(message):
+    """
+    :doc: other
+
+    The default implementation of :func:`renpy.notify`.
     """
 
     hide_screen('notify')
@@ -2979,7 +3165,6 @@ def set_physical_size(size):
     renpy.game.preferences.fullscreen = False
 
     if get_renderer_info()["resizable"]:
-        renpy.display.draw.quit()
         renpy.display.interface.set_mode(size)
         renpy.display.interface.last_resize = size
 
@@ -2996,7 +3181,6 @@ def reset_physical_size():
     renpy.game.preferences.fullscreen = False
 
     if get_renderer_info()["resizable"]:
-        renpy.display.draw.quit()
         renpy.display.interface.set_mode((renpy.config.screen_width, renpy.config.screen_height))
 
 
@@ -3441,7 +3625,7 @@ def play(filename, channel=None, **kwargs):
     if channel is None:
         channel = renpy.config.play_channel
 
-    renpy.audio.music.play(filename, channel=channel, **kwargs)
+    renpy.audio.music.play(filename, channel=channel, loop=False, **kwargs)
 
 
 def get_editable_input_value():
@@ -3588,3 +3772,14 @@ def get_say_image_tag():
         return None
 
     return renpy.store._side_image_attributes[0]
+
+
+def is_skipping():
+    """
+    :doc: other
+
+    Returns True if Ren'Py is currently skipping (in fast or slow skip mode),
+    or False otherwise.
+    """
+
+    return not not renpy.config.skipping
